/*---------------------------------------------------------------------------*\
 ##   ####  ######     |
 ##  ##     ##         | Copyright: ICE Stroemungsfoschungs GmbH
 ##  ##     ####       |
 ##  ##     ##         | http://www.ice-sf.at
 ##   ####  ######     |
-------------------------------------------------------------------------------
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2008 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is based on OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Contributors/Copyright:
    2012-2013 Bernhard F.W. Gschaider <bgschaid@ice-sf.at>

 SWAK Revision: $Id$
\*---------------------------------------------------------------------------*/

#include "gameOfLifeNFunction.H"
#include "FieldValueExpressionDriver.H"

#include "addToRunTimeSelectionTable.H"

#include "emptyFvPatchField.H"
#include "symmetryFvPatchField.H"

namespace Foam {

defineTypeNameAndDebug(gameOfLifeNFunction,0);
addNamedToRunTimeSelectionTable(FieldValuePluginFunction, gameOfLifeNFunction , name, gameOfLife_Nstep);

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

gameOfLifeNFunction::gameOfLifeNFunction(
    const FieldValueExpressionDriver &parentDriver,
    const word &name,
    const string &arguments
):
    gameOfLifeFunction(
        parentDriver,
        name,
        arguments
    ),
    nIterations_(-1)
{
}

gameOfLifeNFunction::gameOfLifeNFunction(
    const FieldValueExpressionDriver &parentDriver,
    const word &name
):
    gameOfLifeFunction(
        parentDriver,
        name,
        string(
            "oldState internalField volScalarField,"
            "nr_of_iterations primitive label"
        )
    ),
    nIterations_(-1)
{
}

// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void gameOfLifeNFunction::setArgument(
    label index,
    const label &value
) {
    if(index==1) {
        nIterations_=value;
        if(value<0) {
            FatalErrorIn("gameOfLifeNFunction::setArgument("
            "label index,"
            "const label &value)")
                << "Got iteration number " << value
                    << ". Should be >=0"
                    << endl
                    << exit(FatalError);
        }
    } else {
        FatalErrorIn("gameOfLifeNFunction::setArgument("
        "label index,"
        "const label &value)")
            << "This should not happen. Expecting index==1. Got " << index
                << endl
                << exit(FatalError);
    }
}

void gameOfLifeNFunction::doEvaluation()
{
    autoPtr<volScalarField> r(
        new volScalarField(oldState())
    );

    for(label i=0;i<nIterations_;i++) {
        r=doStep(r());
    }

    result().setObjectResult(r);
}

// * * * * * * * * * * * * * * * Friend Operators  * * * * * * * * * * * * * //

} // namespace

// ************************************************************************* //
