/*---------------------------------------------------------------------------*\
 ##   ####  ######     |
 ##  ##     ##         | Copyright: ICE Stroemungsfoschungs GmbH
 ##  ##     ####       |
 ##  ##     ##         | http://www.ice-sf.at
 ##   ####  ######     |
-------------------------------------------------------------------------------
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2008 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is based on OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Contributors/Copyright:
    2012-2013 Bernhard F.W. Gschaider <bgschaid@ice-sf.at>

 SWAK Revision: $Id$
\*---------------------------------------------------------------------------*/

#include "gameOfLifeRulesFunction.H"
#include "FieldValueExpressionDriver.H"

#include "addToRunTimeSelectionTable.H"

#include "emptyFvPatchField.H"
#include "symmetryFvPatchField.H"

namespace Foam {

defineTypeNameAndDebug(gameOfLifeRulesFunction,0);
addNamedToRunTimeSelectionTable(FieldValuePluginFunction, gameOfLifeRulesFunction , name, gameOfLife_rules_step);

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

gameOfLifeRulesFunction::gameOfLifeRulesFunction(
    const FieldValueExpressionDriver &parentDriver,
    const word &name,
    const string &arguments
):
    gameOfLifeFunction(
        parentDriver,
        name,
        arguments
    )
{
}

gameOfLifeRulesFunction::gameOfLifeRulesFunction(
    const FieldValueExpressionDriver &parentDriver,
    const word &name
):
    gameOfLifeFunction(
        parentDriver,
        name,
        string(
            "oldState internalField volScalarField,"
            "surviveSlashBirth primitive string"
        )
    )
{
}

// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void gameOfLifeRulesFunction::setArgument(
    label index,
    const string &rules
) {
    if(index==1) {
        survivingNumbers_.clear();
        birthNumbers_.clear();
        std::string::size_type slashPos=rules.find("/");
        if(slashPos==std::string::npos) {
            FatalErrorIn("gameOfLifeRulesFunction::setArgument("
            "label index,"
            "const string &value)")
                << "Rule string must be of the form "
                    << "'<surviving>/<birth>'. No slash found"<< index
                    << endl
                    << exit(FatalError);
        }
        string survive=rules.substr(0,slashPos);
        string birth  =rules.substr(slashPos+1,std::string::npos);
        Dbug << "Survive: " << survive << " Birth: " << birth << endl;
        for(std::string::size_type i=0;i<survive.length();i++) {
            char c=survive[i];
            if(c<'0' || c>'8') {
                FatalErrorIn("gameOfLifeRulesFunction::setArgument("
                "label index,"
                "const string &value)")
                    << "Only 0 to 8 neighbours allowed in rules"<< index
                        << endl
                        << exit(FatalError);
            }
            survivingNumbers_.insert(label(c-'0'));
        }
        for(std::string::size_type i=0;i<birth.length();i++) {
            char c=birth[i];
            if(c<'0' || c>'8') {
                FatalErrorIn("gameOfLifeRulesFunction::setArgument("
                "label index,"
                "const string &value)")
                    << "Only 0 to 8 neighbours allowed in rules"<< index
                        << endl
                        << exit(FatalError);
            }
            birthNumbers_.insert(label(c-'0'));
        }
        Dbug << "Survive: " << survivingNumbers_
            << " Birth: " << birthNumbers_ << endl;
    } else {
        FatalErrorIn("gameOfLifeRulesFunction::setArgument("
        "label index,"
        "const string &value)")
            << "This should not happen. Expecting index==1. Got " << index
                << endl
                << exit(FatalError);
    }
}

// * * * * * * * * * * * * * * * Friend Operators  * * * * * * * * * * * * * //

} // namespace

// ************************************************************************* //
